PROTO = {
    'spaceone.repository.api.v1.repository': ['Repository'],
    'spaceone.repository.api.v1.plugin': ['Plugin'],
    'spaceone.repository.api.v1.schema': ['Schema'],
    'spaceone.repository.api.v1.policy': ['Policy'],
}
